# -*- coding: utf-8 -*-
#
#  This file is part of django-email-change.
#
#  django-email-change adds support for email address change and confirmation.
#
#  Development Web Site:
#    - http://www.codetrax.org/projects/django-email-change
#  Public Source Code Repository:
#    - https://source.codetrax.org/hgroot/django-email-change
#
#  Copyright 2010 George Notaras <gnot [at] g-loaded.eu>
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

from django import forms
from django.db.models.loading import cache


class EmailChangeForm(forms.Form):
    
    # Upon GET requests the initial values of the ``username`` and ``email_old``
    # hidden fields should be set in the ``email_change_view``.
    username = forms.CharField(max_length=30, widget=forms.HiddenInput)
    email_old = forms.CharField(max_length=75, widget=forms.HiddenInput)
    
    email = forms.EmailField(label='New E-mail', max_length=75)
    
    def clean(self):
        """By default, Django's User.email field is not unique. Therefore,
        here we just check whether the new email is not the same as the user's
        current email.
        
        """
        cleaned_data = self.cleaned_data
        username = cleaned_data.get('username')
        email_old = cleaned_data.get('email_old')
        email = cleaned_data.get('email')
        
        if not username or not email_old:
            raise forms.ValidationError('Invalid form data. Please try again.')
        
        User = cache.get_model('auth', 'User')
        
        try:
            User.objects.get(
                username__exact=username, email__exact=email_old)
        except User.DoesNotExist:
            raise forms.ValidationError('Invalid form data. Please try again.')
        
        # Check if the new email address is different from the old email address.
        if email == email_old:
            self._errors['email'] = self.error_class(['New email address cannot be the same as your current email address'])
        
        return cleaned_data



